@@ -1,3 +1,8 @@
+/*
+ * ./configure invocation
+ */
+#undef CONFIGINVOCATION
+
 /*
  * Socket used to communicate
  */
@@ -33,11 +38,6 @@
  */
 #undef MAXMEMB
 
-/*
- * Number of child processes
- */
-#undef MAXGRP
-
 /*
  * TCP connection timeout
  */
@@ -52,3 +52,8 @@
  * use tcp_wrappers libwrap
  */
 #undef USE_LIBWRAP
+
+/*
+ * use ansi prototypes/decls
+ */
+#undef USE_ANSI_PROTO
