@@ -1,5 +1,5 @@
 /*
- *  $Id: master.h,v 5.12 2002-01-21 02:48:33-08 bryan Exp $
+ *  $Id: master.h,v 5.14 2002-02-25 14:00:38-08 bryan Exp $
  *
  *  Copyright conserver.com, 2000
  *
@@ -37,5 +37,10 @@
 /*
  * stuff the master process needs
  */
+#if USE_ANSI_PROTO
+extern void Master();
+extern void SignalKids(int);
+#else
 extern void Master();
 extern void SignalKids();
+#endif
